#!/usr/bin/env python
import user
import importer
import base

import sys

class ScandalDoc (base.Base):
  def __init__(self, clArgs = None, argDB = None):
    return base.Base.__init__(self, clArgs, argDB)

  def clear(self):
    self.debugger       = None
    self.parser         = None
    self.resolver       = None
    self.merge          = None
    self.printers       = {}
    self.outputFiles    = []
    return

  def importModule(moduleName):
    '''Import the named module, and return the module object'''
    module     = __import__(moduleName)
    components = moduleName.split('.')
    for comp in components[1:]:
      module = getattr(module, comp)
    return module
  importModule = staticmethod(importModule)

  def setupArgDB(self, argDB, clArgs):
    '''Setup all local arguments, and process the command line'''
    import nargs

    argDB.setType('debug',        nargs.ArgBool(None, 0, 'Flag to spawn the debugger'),        forceLocal = 1)
    argDB.setType('noStackTrace', nargs.ArgBool(None, 1, 'Suppress the stack trace on error'), forceLocal = 1)
    argDB.setType('includes',     nargs.Arg(None, [], 'List of include files',            isTemporary = 1), forceLocal = 1) # TODO: Should be a list
    argDB.setType('printer',      nargs.Arg(None, [], 'Printer class or list of classes', isTemporary = 1), forceLocal = 1)
    argDB.setType('filename',     nargs.Arg(None, None, 'filename to save SIDL to'),        forceLocal = 1)
    
    base.Base.setupArgDB(self, argDB, clArgs)
    return argDB

  def setupDebug(self):
    if not self.argDB['debug']: return
    import SIDL.Loader

    try:
      import PETSc.Debug.Debugger
    except ImportError, e:
      raise RuntimeError('Could not load Petsc debugger module: '+str(e))

    cls      = 'PETSc.DebugI.GDB.Debugger'
    debugger = PETSc.Debug.Debugger.Debugger(SIDL.Loader.createClass(cls))
    if debugger:
      debugger.setProgram('/usr/local/python/bin/python')
      debugger.setUseXterm(1)
      debugger.setDebugger('/usr/local/gdb-old/bin/gdb')
      debugger.attach()
    else:
      raise RuntimeError('Could not load debugger: '+cls)
    return debugger

  def setupParser(self):
    '''Create the SIDL parser'''
    import SIDL.Loader
    import SIDLLanguage.Parser

    cls    = 'ANL.SIDLCompilerI.SIDLCompiler'
    parser = SIDLLanguage.Parser.Parser(SIDL.Loader.createClass(cls))
    if not parser:
      raise RuntimeError('Unable to instantiate parser: '+cls)
    return parser

  def loadVisitor(self, cls, defaultClass = None):
    '''Create a visitor of class cls. If defaultClass is specified, one will be created on failure to load a cls.'''
    import SIDL.Loader
    import SIDLLanguage.Visitor

    visitor = SIDLLanguage.Visitor.Visitor(SIDL.Loader.createClass(cls))
    if not visitor:
      if defaultClass:
        visitor = SIDLLanguage.Visitor.Visitor(SIDL.Loader.createClass(defaultClass))
        if not visitor:
          raise RuntimeError('Unable to instantiate visitor: '+cls+' or '+defaultClass)
      else:
        raise RuntimeError('Unable to instantiate visitor: '+cls)
    return visitor

  def getPrinter(self, cls):
    '''Return the printer with class name cls. If it has not been loaded already, create it.'''
    if not cls in self.printers:
      import SIDLLanguage.FileOutput

      try:
        printer = SIDLLanguage.FileOutput.FileOutput(self.loadVisitor(cls))
        print 'Loaded printer '+cls
      except Exception, e:
        raise RuntimeError('Could not load printer '+cls+': '+str(e))
      self.printers[cls] = printer
    return self.printers[cls]

  def setupPrinters(self):
    '''Load and configure all printers'''
    if len(self.argDB['printer']):
      printerClasses = self.argDB['printer']
      if not isinstance(printerClasses, list):
        printerClasses = [printerClasses]
      for printerClass in printerClasses:
        printer = self.getPrinter(printerClass)

        if 'filename' in self.argDB:
          try:
            import ANL.SIDL.PrettyPrinterBabel
            p = ANL.SIDL.PrettyPrinterBabel.PrettyPrinterBabel(printer)
            if p:
              p.setFilename(self.argDB['filename'])
          except:
            pass

    return self.printers

  def checkRun(self):
    '''Perform sanity checks before the run'''
    # Check for the loader and parser
    try:
      import SIDL.Loader
      import SIDLLanguage.Parser
    except ImportError, e:
      raise RuntimeError('Cannot locate libraries for SIDL parser: '+str(e))
    return 1

  def setupRun(self):
    '''Construct all components necessary for the run'''
    self.debugger = self.setupDebug()
    self.parser   = self.setupParser()
    if len(self.argDB['printer']):
      self.resolver = self.loadVisitor('ANL.CodeGenerationI.Resolver')
      self.merge    = self.loadVisitor('ANL.SIDL.PackageMerge')
    self.setupPrinters()
    return

  def processIncludes(self):
    '''Read the include filenames from the argument database and return a list of AST objects'''
    includes = []
    for include in self.argDB['includes']:
      includes.append(self.parser.parseFile(include))
    return includes

  def processTargets(self):
    '''Read the target filenames from the argument database'''
    if self.argDB.target == ['default']:
      print 'Usage: '+sys.argv[0]+' [options] <sidl files>'
      sys.exit(1)
    # Take only unique filenames
    trees = []
    self.argDB.target.reverse()
    files = dict([t for t in zip(self.argDB.target, range(len(self.argDB.target)))]).items()
    files.sort(lambda a, b: a[1].__cmp__(b[1]))
    files.reverse()
    for target, num in files:
      trees.append(self.parser.parseFile(target))
    return trees

  def run(self):
    '''Merge all includes and targets into a forest, over which all printers are run'''
    self.clear()
    if not self.checkRun(): return
    self.setupRun()
    includes = self.processIncludes()
    trees    = self.processTargets()
    trees   += includes
    includes = trees

    import SIDL.BaseException
    import SIDLLanguage.ASTException

    if not len(self.printers): return
    try:
      # Resolve all references
      self.resolver.setRepository(includes)
      for tree in trees:
        tree.accept(self.resolver)
      # Merge trees into forest
      for tree in trees:
        tree.accept(self.merge)
      # Run printers on forest
      for tree in self.merge.getRepository():
        for visitor in self.printers.values():
          tree.accept(visitor)
          if visitor.isInstanceOf('SIDLLanguage.FileOutput'):
            for f in SIDLLanguage.FileOutput.FileOutput(visitor).getOutputFiles():
              if not f in self.outputFiles: self.outputFiles.append(f)
    except (SIDL.BaseException.Exception, SIDLLanguage.ASTException.Exception), e:
      print e.getMessage()
      if not self.argDB['noStackTrace']:
        import traceback
        sys.stderr.write(e.getStackTrace())
        print traceback.print_tb(sys.exc_info()[2])
      sys.exit(1)
    return

if __name__ == '__main__': ScandalDoc(sys.argv[1:]).run()
