#!/usr/bin/env python

# ----------------------------------------------------------------------
# sidllex.py
#
# A lexer for SIDL.
# ----------------------------------------------------------------------

import lex

# Reserved words
reserved = (
    'ABSTRACT', 'ARRAY', 'CLASS', 'ENUM', 'EXTENDS', 'FINAL', 'IMPORT', 
    'IN', 'INOUT', 'IMPLEMENTSALL', 'IMPLEMENTS', 'INTERFACE', 'LOCAL', 'ONEWAY', 
    'OUT', 'PACKAGE', 'STATIC', 'THROWS', 'VERSION', 'BOOL', 'CHAR', 
    'DCOMPLEX', 'DOUBLE', 'FCOMPLEX', 'FLOAT', 'INT', 'LONG', 'OPAQUE', 'REPLICATE', 'STRING', 'VOID'
    )

tokens = reserved + (
    # Literals (identifier, integer constant, float constant, string constant, char const)
    'SCOPEDID', 'ID', 'TYPEID', 'VERSION_STRING', 'ICONST', 'ICONST_ALT',
    #'FCONST', 'SCONST', 'CCONST', 
    
    # Operators (<, >)
    'LT', 'GT',

    # Assignment
    'EQUALS',

    # Delimeters ( ) [ ] { } , . ;
    'LPAREN', 'RPAREN',
    'LBRACKET', 'RBRACKET',
    'LBRACE', 'RBRACE',
    'COMMA', 'SEMI',

    # Comments
    'COMMENT',
    )

# Completely ignored characters
t_ignore           = ' \t\x0c'

# Newlines
def t_NEWLINE(t):
    r'\n+'
    t.lineno += t.value.count("\n")

# Implements-all
def t_IMPLEMENTSALL(t):
    r'implements-all'
    return t

# Templates
t_LT               = r'<'
t_GT               = r'>'

# Assignment
t_EQUALS           = r'='

# Delimeters
t_LPAREN           = r'\('
t_RPAREN           = r'\)'
t_LBRACKET         = r'\['
t_RBRACKET         = r'\]'
t_LBRACE           = r'\{'
t_RBRACE           = r'\}'
t_COMMA            = r','
t_SEMI             = r';'
t_COLON            = r':'
t_ELLIPSIS         = r'\.\.\.'

# Identifiers and reserved words

reserved_map = { }
for r in reserved:
    reserved_map[r.lower()] = r

def t_SCOPEDID(t):
    r'[A-Za-z_][\w_-]*(\.[A-Za-z_][\w_-]*)+'
    t.type = reserved_map.get(t.value,"SCOPEDID")
    return t

def t_ID(t):
    r'[A-Za-z_][\w_-]*'
    t.type = reserved_map.get(t.value,"ID")
    return t

# Version literals
def t_VERSION_STRING(t):
    r'(\d+)((\.\d+))+'
    return t

def t_ICONST_ALT(t):
    r'\d+'
    return t

# Integer literal
t_ICONST = r'[+-]?\d+([uU]|[lL]|[uU][lL]|[lL][uU])?'

# Floating literal
#t_FCONST = r'((\d+)(\.\d+)(e(\+|-)?(\d+))? | (\d+)e(\+|-)?(\d+))([lL]|[fF])?'

# String literal
#t_SCONST = r'\"([^\\\n]|(\\.))*?\"'

# Character constant 'c' or L'c'
#t_CCONST = r'(L)?\'([^\\\n]|(\\.))*?\''

# Comments 
def t_COMMENT(t):
    r'/\*(.|\n)*?\*/|//(.*)|/\*[^*]*\*+([^/*][^*]*\*+)*/|"(\\.|[^"\\])*"'
    return t

#t.lineno += t.value.count('\n')

# Preprocessor directive (ignored -- need to change this, save them later - BRN)
def t_preprocessor(t):
    r'\#(.)*?\n'
    t.lineno += 1
    
def t_error(t):
    print "Illegal character %s [line %d]" % (repr(t.value[0]),t.lineno)
    t.skip(1)
    
lexer = lex.lex(optimize=1)
if __name__ == "__main__":
    lex.runmain(lexer)
